from cifsdk.feed.fqdn import Fqdn
from cifsdk.feed.ipv4 import Ipv4
from cifsdk.feed.ipv6 import Ipv6
from cifsdk.feed.url import Url
from cifsdk.feed.email import Email
from cifsdk.feed.sha1 import Sha1

plugins = {
    'ipv4': Ipv4,
    'ipv6': Ipv6,
    'fqdn': Fqdn,
    'url': Url,
    'email': Email,
    'sha1': Sha1
}


# http://stackoverflow.com/a/456747
def factory(name):
    if name in plugins:
        return plugins[name]
    else:
        return None


def tag_contains_whitelist(data):
    for d in data:
        if d == 'whitelist':
            return True



PERM_WHITELIST = []


class Email(object):

    def __init__(self):
        self.wl = set()
        for w in PERM_WHITELIST:
            self.wl.add(w)

    def match_whitelist(self, wl, d):
        bits = d.split('.')

        for i, b in enumerate(bits):
            if '.'.join(bits) in wl:
                return True
            bits.pop(0)

    # https://github.com/jsommers/pytricia
    def process(self, data, whitelist):

        wl = self.wl

        for w in whitelist:
            wl.add(w['observable'])

        rv = []
        for x in data:
            if not self.match_whitelist(wl, x['observable']):
                rv.append(x)

        return rv






from cifsdk.format.cifjson import Json


def test_format_json():
    data = [{
            'observable': "example.com",
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z'
            },
            {
            'observable': "example.com",
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z'
            },
            {
            'observable': "example.com",
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z'
            }]

    print(Json(data))
    assert Json(data)

if __name__ == '__main__':
    test_format_json()



import logging
import json
import time
import datetime
import re
import pytricia
from pprint import pprint

TLP = "amber"
GROUP = "everyone"

RE_IPV4 = re.compile("^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}")
RE_FQDN = re.compile("^(?:[0-9a-zA-Z-]{1,63}\.)+[a-zA-Z]{2,63}$")
RE_URL = re.compile("^(http|https|smtp|ftp|sftp):\/\/")
RE_URL_BROKEN = re.compile("^([a-z0-9.-]+[a-z]{2,63}|\b(?:\d{1,3}\.){3}\d{1,3}\b)(:(\d+))?\/+")

IPV4_PRIVATE = pytricia.PyTricia()
IPV4_PRIVATE_NETS = [
    "0.0.0.0/8",
    "10.0.0.0/8",
    "127.0.0.0/8",
    "192.168.0.0/16",
    "169.254.0.0/16",
    "192.0.2.0/24",
    "224.0.0.0/4",
    "240.0.0.0/5",
    "248.0.0.0/5"
]

for x in IPV4_PRIVATE_NETS:
    IPV4_PRIVATE[x] = True


class Observable(object):

    def __init__(self, observable=None, otype=None, tlp=TLP,
                 reporttime=datetime.datetime.fromtimestamp(time.time()).strftime("%Y-%m-%dT%H:%M:%SZ"),
                 provider=None, group=GROUP, protocol=None, portlist=None, tags=[], asn=None,
                 asn_desc=None, cc=None, application=None, reference=None, reference_tlp=None,
                 raw=[], confidence=None, logger=logging.getLogger(
                __name__)):
        """
        :param observable: string - the observable (ipv4 address, ipv6 address, fqdn, url, hash)
        :param otype: string - observable type (ex: ipv4, ipv6, fqdn, url, email)
        :param tlp: string - Traffic Light Protocol (ex: Red, Amber, Green, White)
        :param reporttime: string - timestamp (Format: YYYY-MM-DDTHH:MM:SSZ)
        :param provider: string - provider (ex: spamhaus.org)
        :param group: string - group membership (ex: everyone)
        :param protocol: string - layer 4 protocol (ex: icmp, tcp, udp)
        :param portlist: string - list of ports (ex: 1,2,445-446)
        :param tags: string - tags describing data (ex: malware, botnet)
        :param asn: int - ASN number (ex: 87)
        :param asn_desc: string - ASN description (ex: Indiana University)
        :param cc: string - Country Code (ex: US)
        :param application: string - Application commonly associated to the port (ex: http, ssh)
        :param reference: string - Reference text (ex: https://www.spamhaus.org/drop/)
        :param reference_tlp: string - Traffic Light Protocol (ex: Red, Amber, Green, White)
        :param raw:
        :param confidence: int - Number describing confidence in observable (0-100)
        :return: object
        """

        self.logger = logger

        if isinstance(tags, basestring):
            tags = tags.split(",")

        self.observable = observable
        self.tlp = tlp
        self.provider = provider
        self.reporttime = reporttime
        self.group = group
        self.otype = otype
        self.protocol = protocol
        self.portlist = portlist
        self.tags = tags
        self.application = application
        self.reference = reference
        self.reference_tlp = reference_tlp
        self.raw = raw
        self.confidence = confidence

        if asn and asn.lower() == 'na':
            asn = None

        self.asn = asn

        if asn_desc and asn_desc.lower() == 'na':
            asn_desc = None

        self.asn_desc = asn_desc
        self.cc = cc

        if not otype:
            self.otype = self.resolve_obj(self.observable)

    def is_private(self):
        if self.otype and self.otype == 'ipv4':
            if IPV4_PRIVATE.get(self.observable):
                return True
        return False

    def resolve_obj(self, observable):
        def _ipv4(s):
            if RE_IPV4.match(s):
                return 1

        def _fqdn(s):
            if RE_FQDN.match(s):
                return 1

        def _url(s):
            if RE_URL.match(s):
                return 1

        if _fqdn(observable):
            return 'fqdn'
        elif _ipv4(observable):
            return 'ipv4'
        elif _url(observable):
            return 'url'

    def __repr__(self):
        o = {
            "observable": self.observable,
            "otype": self.otype,
            "tlp": self.tlp,
            "reporttime": self.reporttime,
            "provider": self.provider,
            "portlist": self.portlist,
            "protocol": self.protocol,
            "tags": ",".join(self.tags),
            "asn": self.asn,
            "asn_desc": self.asn_desc,
            "cc": self.cc,
            "group": self.group,
            "raw": self.raw,
            "confidence": self.confidence
        }
        if self.logger.getEffectiveLevel() == 10:
            return json.dumps(o, indent=4, sort_keys=True)
        else:
            return json.dumps(o, sort_keys=True)




def tag_contains_whitelist(data):
    for d in data:
        if d == 'whitelist':
            return True


class Url(object):

    def __init__(self):
        pass

    def process(self, data, whitelist):
        wl = set()
        for x in whitelist:
            wl.add(x['observable'])

        rv = []
        for x in data:
            if tag_contains_whitelist(x['tags']):
                continue

            if x['observable'] not in wl:
                rv.append(x)

        return rv






import pytest


from cifsdk.feed.fqdn import Fqdn
from pprint import pprint

data = [
    {
        'observable': 'example.com',
        'tags': 'botnet',
    },
    {
        'observable': 'test.example.com',
        'tags': 'malware',
    },
    {
        'observable': 'test.test.com',
        'tags': 'scanner'
    },
    {
        'observable': 'amazon.com',
        'tags': 'scanner'
    },
    {
        'observable': 'test.amazon.com',
        'tags': 'scanner'
    },
        {
        'observable': 'csirtgadgets.org',
        'tags': 'scanner'
    },
    {
        'observable': 'evil.org',
        'tags': 'scanner'
    },
    {
        'observable': 'gmail-smtp-in.l.google.com',
        'tags': 'malware'
    }
]

whitelist = [
    {
        'observable': 'example.com',
    },
    {
        'observable': 'google.com',
    }
]


def test_feed_fqdn_whitelist():
    x = Fqdn()

    y = x.process(data, whitelist)

    s = set()

    for xx in x.process(data, whitelist):
        s.add(xx['observable'])

    assert 'example.com' not in s
    assert 'csirtgadgets.org' in s
    assert 'amazon.com' not in s

    # need to create the trie mod
    assert 'test.example.com' not in s
    assert 'test.amazon.com' not in s
    assert 'amazon.com' not in s
    assert 'evil.org' in s

    assert 'gmail-smtp-in.l.google.com' not in s


import os

LOG_FORMAT = '%(asctime)s - %(levelname)s - %(name)s[%(lineno)s] - %(message)s'

REMOTE_ADDR = 'https://localhost'
REMOTE_ADDR = os.environ.get('CIF_REMOTE_ADDR', REMOTE_ADDR)

LIMIT = 500
LIMIT = os.environ.get('CIF_LIMIT', LIMIT)

FEED_LIMIT = 50000
FEED_LIMIT = os.environ.get('CIF_FEED_LIMIT', FEED_LIMIT)

FEED_CONFIDENCE = 85
FEED_CONFIDENCE = os.environ.get('CIF_FEED_CONFIDENCE', FEED_CONFIDENCE)

WHITELIST_LIMIT = 25000
WHITELIST_LIMIT = os.environ.get('CIF_WHITELIST_LIMIT', WHITELIST_LIMIT)

WHITELIST_CONFIDENCE = os.environ.get('CIF_WHITELIST_CONFIDENCE', 25)

PROXY = os.environ.get('CIF_PROXY')

TOKEN = os.environ.get('CIF_TOKEN')

FIELDS = ['tlp', 'group', 'lasttime', 'reporttime', 'observable', 'otype', 'cc', 'asn', 'asn_desc', 'confidence', 'description',
          'tags', 'rdata', 'rtype', 'provider', 'altid', 'altid_tlp']
FIELDS = os.environ.get('CIF_FIELDS', FIELDS)

PINGS=4



from cifsdk.format.cifsnort import Snort

import re
RULE_PATTERN = r'^alert (TCP|UDP|IP) (\S+) (\S+) -> ([^,]+)\s(\S+)\s\([^.]+\)'

def test_format_snort():
    data = [
        {
            'observable': "example.com",
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z',
            'otype': 'fqdn',
            'tags': ['botnet']
        },
        {
            'observable': "http://example.com/1234.htm",
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z',
            'otype': 'url',
            'tags': ['botnet']
        },
        {
            'observable': "https://example.com/1234.htm",
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z',
            'otype': 'url',
            'tags': ['botnet']
        },
        {
            'observable': "192.168.1.1",
            'portlist': 8888,
            'protocol': 6,
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z',
            'otype': 'ipv4',
            'tags': ['botnet']
        }
    ]

    text = str(Snort(data))
    assert text
    assert re.findall(RULE_PATTERN, text)


if __name__ == '__main__':
    test_format_snort()



from cifsdk.format.plugin import Plugin
import csv

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


class Csv(Plugin):

    def __init__(self, *args, **kwargs):
        super(Csv, self).__init__(*args, **kwargs)

    def __repr__(self):
        si = StringIO()
        cw = csv.writer(si, quoting=csv.QUOTE_ALL)
        cw.writerow(self.cols)
        for obs in self.data:
            r = []
            for c in self.cols:
                y = obs.get(c) or ''
                if type(y) is list:
                    y = ','.join(y)
                if type(y) == int:
                    y = str(y)
                if type(y) == float:
                    y = str(y)

                r.append(y.encode('utf-8'))
            cw.writerow(r)
        return si.getvalue().strip('\r\n')



import pytricia
import logging
import ipaddress
import sys

from pprint import pprint

PERM_WHITELIST = [
    "0.0.0.0/8",
    "10.0.0.0/8",
    "127.0.0.0/8",
    "192.168.0.0/16",
    "169.254.0.0/16",
    "192.0.2.0/24",
    "224.0.0.0/4",
    "240.0.0.0/5",
    "248.0.0.0/5",
]

LARGEST_PREFIX = '8'


def tag_contains_whitelist(data):
    for d in data:
        if d == 'whitelist':
            return True


def _normalize(i):
    bits = i.split('.')

    rv = []
    for b in bits:
        if len(b) > 1 and b.startswith('0') and not b.startswith('0/'):
            b = b[1:]
        rv.append(b)

    return '.'.join(rv)


class Ipv4(object):

    def __init__(self):
        pass

    # https://github.com/jsommers/pytricia
    def process(self, data, whitelist=[]):
        wl = pytricia.PyTricia()
        for x in PERM_WHITELIST:
            wl[x] = True

        for y in whitelist:
            y = str(_normalize(y['observable']))
            if not '/' in y: # weird bug work-around it'll insert 172.16.1.60 with a /0 at the end??
                y = '{}/32'.format(y)
            wl[y] = True

        # this could be done with generators...
        rv = []

        for y in data:
            if tag_contains_whitelist(y['tags']):
                continue

            y['observable'] = _normalize(y['observable'])
            try:
                if sys.version_info.major < 3:
                    ipaddress.ip_network(unicode(y['observable']))
                else:
                    ipaddress.ip_network(y['observable'])
                if str(y['observable']) not in wl:
                    rv.append(y)
            except ValueError as e:
                print(e)
                print('skipping invalid address: %s' % y['observable'])

        return rv







import pytest


from cifsdk.feed.ipv6 import Ipv6
from pprint import pprint
import logging

data = [
    {
        'observable': '2001:4860:4860::8888',
        'tags': 'botnet',
    }
]

whitelist = [
    {
        'observable': '2001:4860:4860::8888',
        'tags': 'whitelist',
    }
]


def test_feed_ipv6_whitelist():
    #pass # https://github.com/jsommers/pytricia/issues/6
    x = Ipv6()
    s = set()
    for xx in x.process(data, whitelist):
        s.add(xx['observable'])

    assert '2001:4860:4860::8888' not in s


from cifsdk.format.plugin import Plugin
import json


class Json(Plugin):

    def __init__(self, *args, **kwargs):
        super(Json, self).__init__(*args, **kwargs)

    def __repr__(self):
        return json.dumps(self.data)


import pytricia
import logging
from pprint import pprint

PERM_WHITELIST = [
    ## TODO -- more
    # http://www.iana.org/assignments/ipv6-multicast-addresses/ipv6-multicast-addresses.xhtml
    # v6
    'FF01:0:0:0:0:0:0:1',
    'FF01:0:0:0:0:0:0:2',
]


def tag_contains_whitelist(data):
    for d in data:
        if d == 'whitelist':
            return True


class Ipv6(object):

    def __init__(self):
        self.logger = logging.getLogger(__name__)
        pass

    def process(self, data, whitelist=[]):
        wl = pytricia.PyTricia()

        [wl.insert(x, True) for x in PERM_WHITELIST]

        [wl.insert(str(y['observable']), True) for y in whitelist]

        rv = []
        for y in data:
            if tag_contains_whitelist(y['tags']):
                continue

            if str(y['observable']) not in wl:
                rv.append(y)

        return rv







import ujson as json
import time
import sys
import os
import os.path
import yaml
import logging
import select
from cifsdk.format import factory as format_factory
from cifsdk.feed import factory as feed_factory

from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter
from signal import signal, SIGPIPE, SIG_DFL
import textwrap
import copy
import arrow
import zlib
import base64

from cifsdk import VERSION, API_VERSION
from cifsdk.constants import REMOTE_ADDR, LIMIT, FEED_CONFIDENCE, WHITELIST_LIMIT, PROXY, FEED_LIMIT, TOKEN, FIELDS
from cifsdk.constants import PINGS, WHITELIST_CONFIDENCE
from cifsdk.utils import setup_logging, read_config
from time import sleep

# https://urllib3.readthedocs.org/en/latest/security.html#disabling-warnings
# http://stackoverflow.com/questions/14789631/hide-userwarning-from-urllib2
import requests
requests.packages.urllib3.disable_warnings()


RETRIES = os.environ.get('CIFSDK_RETRIES', 4)
RETRY_SLEEP = 5


class Client(object):

    def __init__(self, token, remote=REMOTE_ADDR, proxy=None, timeout=300, verify_ssl=True, nowait=False):
        """
        Initiates a client object

        :param token: <cif token> (ex: 6e10366ce0a25227aac810b4058c3712d30d3848f4d5d8f586658178a65c67df)
        :param remote: server location (ex: https://localhost)
        :param proxy: proxy server location
        :param timeout: seconds for client timeout (default: 300)
        :param no_verify_ssl: turn off TLS verification (default: False)
        :param nowait: batch submissions on the server, do not wait for returned submission id's. Best to
               to use this if submitting more than 100 records at a time. (default: False)
        :return: object
        """
        
        self.logger = logging.getLogger(__name__)
        self.remote = remote
        self.token = str(token)
        self.proxy = proxy
        self.timeout = timeout
        self.verify_ssl = verify_ssl
        
        self.session = requests.session()
        self.session.headers["Accept"] = "application/vnd.cif.v{0}+json".format(API_VERSION)
        self.session.headers['User-Agent'] = "py-cifsdk/{0}".format(VERSION)
        self.session.headers['Authorization'] = "Token token={0}".format(self.token)
        self.session.headers['Content-Type'] = 'application/json'

        self.nowait = nowait
    
    def search(self, query=None, filters={}, limit=None, nolog=None, sort='lasttime', sort_direction='ASC', decode=True):
        """returns search result set based on either query or filters

        :param query: a single observable (ex: example.com, 192.168.1.1, ...)
        :param filters: filter results by various attributes: https://github.com/csirtgadgets/massive-octo-spice/wiki/API
        :param limit: limit return results
        :param nolog: do NOT log query
        :param sort: sort result set (default: 'lasttime')
        :param decode: decode the results from JSON (default: yes)
        :return: list of dicts (observables)
        """
        filters['limit'] = limit
        filters['nolog'] = nolog
        filters['gzip'] = 1

        if query:
            filters['observable'] = query

        if filters.get('observable'):
            filters['observable'] = filters['observable'].lower()
        
        uri = self.remote + '/observables'

        if filters.get('tags') and type(filters.get('tags')) is list:
            filters['tags'] = ','.join(filters['tags'])

        if filters.get('cc'):
            filters['cc'] = filters['cc'].lower()
            
        self.logger.debug('uri: %s' % uri)
        self.logger.debug('params: %s', json.dumps(filters))

        self.logger.info('searching...')

        body = self.session.get(uri, params=filters, verify=self.verify_ssl)
        self.logger.debug('status code: ' + str(body.status_code))

        if body.status_code >= 422:
            max_retries = RETRIES
            while max_retries > 0:
                self.logger.warn('{} found, retrying in {}s... {} tries remaining'.format(body.status_code, RETRY_SLEEP,
                                                                                          max_retries))
                sleep(RETRY_SLEEP)
                body = self.session.get(uri, params=filters, verify=self.verify_ssl)
                if body.status_code == 200:
                    break
                else:
                    max_retries -= 1

        if body.status_code > 299:
            self.logger.warning('request failed: %s' % str(body.status_code))
            raise SystemExit

        s = (int(body.headers['Content-Length']) / 1024 / 1024)
        self.logger.info('processing {0} megs'.format(s))

        ret = body.content
        if not ret.startswith('['):
            self.logger.info('trying to decompress...')
            # http://stackoverflow.com/a/2695575
            ret = base64.b64decode(ret)
            try:
                ret = zlib.decompress(ret)
            except zlib.error as e:
                # v2 gzip style
                ret = zlib.decompress(ret, 16+zlib.MAX_WBITS)

        if decode:
            self.logger.info('decoding...')
            ret = json.loads(ret)

            if type(ret) == dict and ret.get('data'):
                ret = ret['data']

            self.logger.info('sorting...')
            if sort_direction == 'DESC':
                ret = sorted(ret, key=lambda o: o[sort], reverse=True)
            else:
                ret = sorted(ret, key=lambda o: o[sort])

        self.logger.debug('returning..')
        return ret

    def submit(self, data):
        """
        Submit records to CIF

        :param data: a single dict or a list of dicts
        :return: list

        list
        [{"observable": "1.1.1.1", "confidence": "85", "tlp": "amber", "group": "everyone", "tags": ["zeus","botnet"],
        "provider": "me.com"}, {"observable": "1.1.1.1", "confidence": "85", "tlp": "amber", "group": "everyone",
        "tags": "malware", "provider": "me.com"}]
        """

        if type(data) == dict:
            data = [data]

        if type(data[0]) != dict:
            raise RuntimeError('submitted data must be a dictionary')

        data = json.dumps(data)

        # TODO - http://docs.python-requests.org/en/latest/user/quickstart/#more-complicated-post-requests
        uri = self.remote + '/observables'

        if self.nowait:
            uri = "{0}?nowait=1".format(uri)
        
        self.logger.debug('uri: %s' % uri)

        body = self.session.post(uri, data=data, verify=self.verify_ssl)
        self.logger.debug('status code: ' + str(body.status_code))
        if body.status_code > 299:
            self.logger.error('request failed: %s' % str(body.status_code))
            self.logger.error(json.loads(body.text).get('message'))
            return None
        
        body = json.loads(body.text)
        return body
    
    def ping(self):
        """
        Ping the server to verify connectivity
        :return: str
        """
        t0 = time.time()
        uri = str(self.remote) + '/ping'
        body = self.session.get(uri, params={}, verify=self.verify_ssl)
        
        self.logger.debug('status code: ' + str(body.status_code))
        if body.status_code > 299:
            self.logger.error('request failed: %s' % str(body.status_code))
            return 'request failed: %s' % str(body.status_code)
        
        t1 = (time.time() - t0)
        self.logger.debug('return time: %.15f' % t1)
        return t1

    def aggregate(self, data, field='observable', sort='confidence', sort_secondary='reporttime'):
        """
        aggregate data
        :param data:
        :param field:
        :param sort:
        :return:
        """
        x = set()
        rv = []
        for d in sorted(data, key=lambda x: x[sort], reverse=True):
            if d[field] not in x:
                x.add(d[field])
                rv.append(d)

        rv = sorted(rv, key=lambda x: x[sort_secondary])
        return rv


def main():

    p = ArgumentParser(
        description=textwrap.dedent('''\
        Example usage:

            $ cif -q 130.201.0.2
            $ cif -q 130.201.0.0/16
            $ cif -q 2001:4860:4860::8888
            $ cif -q example.com
            $ cif -q 'http://www.example.com'
            $ cif -q 'john@example.com'
            $ cif -q bf9d457bcd702fe836201df1b48c0bec

            $ cif --tags botnet,zeus -c 85
            $ cif --application vnc,ssh --asns 1234 --cc RU,US
            $ cif -q example.com --tags botnet,zeus -c 85 --limit 50

            $ cif --otype ipv4 --aggregate observable --today

            $ cif --feed --otype ipv4 -c 85 -f csv
            $ cif --feed --otype fqdn -c 95 --tags botnet -f csv
            $ cif --feed --otype url -c 75 --today -f csv
        '''),
        formatter_class=RawDescriptionHelpFormatter,
        prog='cif'
    )

    # options
    p.add_argument("-v", "--verbose", dest="verbose", action="store_true", help="logging level: INFO")
    p.add_argument('-d', '--debug', dest='debug', action="store_true", help="logging level: DEBUG")
    p.add_argument('-V', '--version', action='version', version=VERSION)
    p.add_argument('--no-verify-ssl', action="store_true", default=False)
    p.add_argument('-R', '--remote',  help="remote api location")
    p.add_argument('-T', '--token', help="specify token",  default=TOKEN)
    p.add_argument('--timeout',  help='connection timeout [default: %(default)s]', default="300")
    p.add_argument('-C', '--config',  help="configuration file [default: %(default)s]",
                   default=os.path.expanduser("~/.cif.yml"))

    p.add_argument('--sortby', help='sort output [default: %(default)s]', default='lasttime')
    p.add_argument('--sortby-direction', help='sortby direction [default: %(default)s]', default='ASC')
    p.add_argument('-f', '--format', help="specify output format [default: %(default)s]", default="table")

    # actions
    p.add_argument('-p', '--ping', action="store_true", help="ping")
    p.add_argument('-s', '--submit', action="store_true", help="submit a JSON object")

    # flags
    p.add_argument('-l', '--limit', help="result limit", default=LIMIT)
    p.add_argument('-n', '--nolog', help='do not log the search', default=None, action="store_true")

    # filters
    p.add_argument('-q', "--query", help="specify a search")
    p.add_argument('--firsttime', help='specify filter based on firsttime timestmap (greater than, '
                                       'format: YYYY-MM-DDTHH:MM:SSZ)')
    p.add_argument('--lasttime', help='specify filter based on lasttime timestamp (less than, format: '
                                      'YYYY-MM-DDTHH:MM:SSZ)')
    p.add_argument('--reporttime', help='specify filter based on reporttime timestmap (greater than, format: '
                                        'YYYY-MM-DDTHH:MM:SSZ)')
    p.add_argument('--reporttimeend', help='specify filter based on reporttime timestmap (less than, format: '
                                           'YYYY-MM-DDTHH:MM:SSZ)')
    p.add_argument("--tags", help="filter for tags")
    p.add_argument('--description', help='filter on description')
    p.add_argument('--otype', help='filter by otype')
    p.add_argument("--cc", help="filter for countrycode")
    p.add_argument('-c', '--confidence', help="specify confidence")
    p.add_argument('--rdata', help='filter by rdata')
    p.add_argument('--provider', help='filter by provider')
    p.add_argument('--asn', help='filter by asn')
    p.add_argument('--tlp', help='filter by tlp')
    p.add_argument('--proxy', help="specify a proxy to use [default %(default)s]", default=PROXY)

    p.add_argument('--feed', action="store_true", help="generate a feed of data, meaning deduplicated and whitelisted")
    p.add_argument('--whitelist-limit', help="specify how many whitelist results to use when applying to --feeds "
                                             "[default %(default)s]", default=WHITELIST_LIMIT)
    p.add_argument('--whitelist-confidence', help='by confidence (greater-than or equal to) [default: %(default)s]',
                   default=WHITELIST_CONFIDENCE)

    p.add_argument('--last-day', action="store_true", help='auto-sets reporttime to 23 hours and 59 seconds ago '
                                                           '(current time UTC) and reporttime-end to "now"')
    p.add_argument('--last-hour', action='store_true', help='auto-sets reporttime to the beginning of the previous full'
                                                            ' hour and reporttime-end to end of previous full hour')
    p.add_argument('--days', help='filter results within last X days')
    p.add_argument('--today', help='auto-sets reporttime to today, 00:00:00Z (UTC)', action='store_true')

    p.add_argument('--aggregate', help="aggregate around a specific field (ie: observable)")

    p.add_argument('--fields', help="specify field list to display [default: %(default)s]", default=','.join(FIELDS))
    p.add_argument('--filename', help='specify output filename [default: STDOUT]')
    p.add_argument('--ttl', help='specify number of pings to send [default: %(default)s]', default=PINGS)
    p.add_argument('--group', help='filter by group(s) (everyone,group1,group2,...)')
    p.add_argument('--application', help='filter based on application field')
    p.add_argument('--id', help='specify an id to retrieve')

    # Process arguments
    args = p.parse_args()
    setup_logging(args)
    logger = logging.getLogger(__name__)

    o = read_config(args)
    options = vars(args)
    for v in options:
        if options[v] is None:
            options[v] = o.get(v)

    if not options.get('token'):
        raise RuntimeError('missing --token')

    verify_ssl = True
    if o.get('no_verify_ssl') or options.get('no_verify_ssl'):
        verify_ssl = False

    cli = Client(options['token'], remote=options['remote'], proxy=options.get('proxy'), verify_ssl=verify_ssl)

    if(options.get('query') or options.get('tags') or options.get('cc') or options.get('rdata') or options.get(
                'otype') or options.get('provider') or options.get('asn') or options.get('description')):
        filters = {}
        if options.get('query'):
            filters['observable'] = options['query']
        if options.get('cc'):
            filters['cc'] = options['cc']

        if options.get('tags'):
            filters['tags'] = options['tags']

        if options.get('description'):
            filters['description'] = options['description']

        if options.get('confidence'):
            filters['confidence'] = options['confidence']
        else:
            if options.get('feed'):
                filters['confidence'] = FEED_CONFIDENCE

        if options.get('firsttime'):
            filters['firsttime'] = options['firsttime']

        if options.get('lasttime'):
            filters['lasttime'] = options['lasttime']

        if options.get('reporttime'):
            filters['reporttime'] = options['reporttime']

        if options.get('reporttimeend'):
            filters['reporttimeend'] = options['reporttimeend']

        if options.get('otype'):
            filters['otype'] = options['otype']

        if options.get('rdata'):
            filters['rdata'] = options['rdata']

        if options.get('nolog'):
            options['nolog'] = 1

        if options.get('provider'):
            filters['provider'] = options['provider']

        if options.get('asn'):
            filters['asn'] = options['asn']

        if options.get('tlp'):
            filters['tlp'] = options['tlp']

        if options.get('group'):
            filters['group'] = options['group']

        if options.get('application'):
            filters['application'] = options['application']

        if options.get('id'):
            filters['id'] = options['id']

        # needs to be MEG'd out.
        if options.get('last_day'):
            now = arrow.utcnow()
            filters['reporttimeend'] = '{0}Z'.format(now.format('YYYY-MM-DDTHH:mm:ss'))
            now = now.replace(days=-1)
            filters['reporttime'] = '{0}Z'.format(now.format('YYYY-MM-DDTHH:mm:ss'))
        elif options.get('last_hour'):
            now = arrow.utcnow()
            filters['reporttimeend'] = '{0}Z'.format(now.format('YYYY-MM-DDTHH:mm:ss'))
            now = now.replace(hours=-1)
            filters['reporttime'] = '{0}Z'.format(now.format('YYYY-MM-DDTHH:mm:ss'))
        elif options.get('today'):
            now = arrow.utcnow()
            filters['reporttime'] = '{0}Z'.format(now.format('YYYY-MM-DDT00:00:00'))

        if options.get('days'):
            now = arrow.utcnow()
            filters['reporttimeend'] = '{0}Z'.format(now.format('YYYY-MM-DDTHH:mm:ss'))
            now = now.replace(days=-int(options['days']))
            filters['reporttime'] = '{0}Z'.format(now.format('YYYY-MM-DDTHH:mm:ss'))

        DAYS=30

        if options.get('feed'):
            if options['limit'] == LIMIT:
                options['limit'] = FEED_LIMIT

            if not options.get('days'):
                now = arrow.utcnow()
                filters['reporttimeend'] = '{0}Z'.format(now.format('YYYY-MM-DDTHH:mm:ss'))
                now = now.replace(days=-DAYS)
                filters['reporttime'] = '{0}Z'.format(now.format('YYYY-MM-DDTHH:mm:ss'))

        ret = cli.search(limit=options['limit'], nolog=options['nolog'], filters=filters, sort=options['sortby'],
                         sort_direction=options['sortby_direction'])

        number_returned = len(ret)

        logger.info('returned: {0} records'.format(number_returned))

        if options.get('aggregate'):
            ret = cli.aggregate(ret, field=options['aggregate'])

        if options.get('feed'):
            wl_filters = copy.deepcopy(filters)
            wl_filters['tags'] = 'whitelist'
            wl_filters['confidence'] = args.whitelist_confidence

            now = arrow.utcnow()
            now = now.replace(days=-DAYS)
            wl_filters['reporttime'] = '{0}Z'.format(now.format('YYYY-MM-DDTHH:mm:ss'))

            wl = cli.search(limit=options['whitelist_limit'], nolog=True, filters=wl_filters)

            f = feed_factory(options['otype'])

            ret = cli.aggregate(ret)

            if len(ret) != number_returned:
                logger.info('aggregation removed: {0} records'.format(number_returned - len(ret)))

            ret = f().process(ret, wl)

        f = format_factory(options['format'])
        if f is None:
            raise SystemError('{0} format not supported, maybe missing a dependency.'.format(options['format']))

        try:
            if len(ret) >= 1:
                ret = f(ret, cols=options['fields'].split(','))
                if args.filename:
                    with open(args.filename, 'w') as F:
                        F.write(str(ret))
                else:
                    signal(SIGPIPE, SIG_DFL)
                    print(ret)
            else:
                logger.info("no results found...")
        except AttributeError as e:
            logger.exception(e)

    elif options.get('ping'):
        for num in range(0, args.ttl):
            ret = cli.ping()
            print("roundtrip: %s ms" % ret)
            select.select([], [], [], 1)
    elif options.get('submit'):

        if not sys.stdin.isatty():
            stdin = sys.stdin.read()
        else:
            logger.error("No data passed via STDIN")
            raise SystemExit

        try:
            data = json.loads(stdin)
            try:
                ret = cli.submit(data)
                print('submitted: {0}'.format(ret))
            except Exception as e:
                logger.error(e)
                raise SystemExit
        except Exception as e:
            logger.error(e)
            raise SystemExit
    else:
        logger.warning('operation not supported')
        p.print_help()
        raise SystemExit

if __name__ == "__main__":
    main()



try:
    import elasticsearch  # 1.9.0
except ImportError:
    print('this module requires the elasticsearch library')
    raise ImportError

from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter
import logging
import textwrap
from elasticsearch import Elasticsearch
from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta
from elasticsearch import helpers
import re
from cifsdk import VERSION
from cifsdk.utils import setup_logging
from pprint import pprint

CONFIDENCE = 50
MONTHS = 12
LIMIT = 5000


def main():

    p = ArgumentParser(
        description=textwrap.dedent('''\
        example usage:
            $ cif-es-reindex
        '''),
        formatter_class=RawDescriptionHelpFormatter,
        prog='cif-es-reindex'
    )

    # options
    p.add_argument("-v", "--verbose", action="store_true", help="logging level: INFO")
    p.add_argument('-d', '--debug', action="store_true", help="logging level: DEBUG")
    p.add_argument('-V', '--version', action='version', version=VERSION)
    p.add_argument('-m', '--months', help='how many months ago to cull [default %(default)s]', default=MONTHS)
    p.add_argument('-c', '--confidence', help='min confidence [default %(default)s]', default=CONFIDENCE)
    p.add_argument('--index-prefix', help='index prefix', default='cif.observables')
    p.add_argument('--dry-run', action="store_true", help='dry run, do not delete')
    p.add_argument('--nodes', default=['localhost:9200'])

    args = p.parse_args()
    setup_logging(args)
    logger = logging.getLogger(__name__)

    end_month = (datetime.today() - relativedelta(months=int(args.months)))
    end_month = end_month.strftime('%Y.%m')

    logger.info('month: {}'.format(end_month))

    es = Elasticsearch(args.nodes, timeout=120, max_retries=10, retry_on_timeout=True)

    body = {
        'query': {
            'filtered': {
                'filter': {
                    'and': [
                        {
                            'range': {
                                'confidence': {'lt': args.confidence}
                            }
                        }
                    ]
                }
            }
        }
    }

    monthlies = es.indices.get_aliases(index='{}-*.*'.format(args.index_prefix)).keys()
    to_cull = {}
    for m in monthlies:
        match = re.search(r"^cif\.observables-(\d{4}\.\d{2})$", m)
        if match.group(1) < end_month:
            to_cull['{}-{}'.format(args.index_prefix, match.group(1))] = '{}-{}'.format(args.index_prefix,
                                                                                        match.group(1))

    # https://www.elastic.co/guide/en/elasticsearch/reference/1.4/docs-delete-by-query.html
    # http://elasticsearch-py.readthedocs.io/en/master/api.html#elasticsearch.Elasticsearch.delete_by_query
    # http://stackoverflow.com/questions/26808239/elasticsearch-python-api-delete-documents-by-query

    for c in to_cull:
        logger.info('culling: {}'.format(c))
        if not args.dry_run:
            rv = helpers.scan(
                es,
                index=c,
                query=body,
                scroll='5m',
                size=LIMIT,
            )

            for r in rv:
                pprint(r)

            # rv = es.delete_by_query(
            #     index=c,
            #     body=body,
            #     wait_for_completion=True
            #
            # )
            # pprint(rv)
            # es.indices.optimize(index=to_cull[c])

if __name__ == "__main__":
    main()




from cifsdk.format.plugin import Plugin
import re

from pprint import pprint
otype = {
    'ipv4': 'ADDR',
    'url': 'URL',
    'fqdn': 'DOMAIN',
    'email': 'EMAIL',
    'md5': 'FILE_HASH',
    'sha1': 'FILE_HASH',
    'sha256': 'FILE_HASH',
}

HEADER = '#' + '\t'.join(['fields', 'indicator', 'indicator_type', 'meta.desc', 'meta.cif_confidence', 'meta.source'])
SEP = '|'

class Bro(Plugin):
    __name__ = 'bro'

    def __init__(self, *args, **kwargs):
        super(Bro, self).__init__(*args, **kwargs)

        self.cols = ['observable', 'otype', 'tags', 'confidence', 'provider']

    def __repr__(self):
        text = []
        for d in self.data:
            r = []
            if d['otype'] is 'url':
                d['observable'] = re.sub(r'(https?\:\/\/)', '', d['observable'])

            for c in self.cols:
                y = d.get(c, '-')
                if type(y) is list:
                    y = SEP.join(y)
                y = str(y)
                if c is 'otype':
                    y = 'Intel::{0}'.format(otype[d[c]])
                r.append(y)
            text.append("\t".join(r))

        text = "\n".join(text)
        
        text = "{0}\n{1}".format(HEADER, text)
        return text



from cifsdk.format.cifjson import Json
from cifsdk.format.cifcsv import Csv
from cifsdk.format.table import Table
from cifsdk.format.bro import Bro
from cifsdk.format.cifsnort import Snort
from cifsdk.format.cifbind import Bind

plugins = {
    'json': Json,
    'table': Table,
    'csv': Csv,
    'bro': Bro,
    'snort': Snort,
    'bind': Bind,
}

try:
    from cifsdk.format.cif_stix import Stix
    plugins['stix'] = Stix
except:
    pass


# http://stackoverflow.com/a/456747
def factory(name):
    if name in plugins:
        return plugins[name]
    else:
        return None



import pkgutil
import logging
import yaml
import os
from cifsdk.constants import LOG_FORMAT


def read_config(args):
    options = {}
    if os.path.isfile(args.config):
        f = file(args.config)
        config = yaml.load(f)
        config = config['client']
        f.close()
        if not config:
            print("Unable to read {} config file".format(args.config))
            raise SystemExit
        for k in config:
            if not options.get(k):
                options[k] = config[k]
    else:
        print("Unable to read {} config file".format(args.config))
        raise SystemExit

    return options


def load_plugin(path, plugin):
    p = None
    for loader, modname, is_pkg in pkgutil.iter_modules([path]):
        if modname == plugin:
            p = loader.find_module(modname).load_module(modname)
            p = p.Plugin

    return p


def setup_logging(args):
    loglevel = logging.WARNING
    if args.verbose:
        loglevel = logging.INFO
    if args.debug:
        loglevel = logging.DEBUG

    console = logging.StreamHandler()
    logging.getLogger('').setLevel(loglevel)
    console.setFormatter(logging.Formatter(LOG_FORMAT))
    logging.getLogger('').addHandler(console)



from prettytable import PrettyTable
from cifsdk.format.plugin import Plugin


class Table(Plugin):

    def __init__(self, *args, **kwargs):
        super(Table, self).__init__(*args, **kwargs)

    def __repr__(self):
        t = PrettyTable(self.cols)
        t.align['provider'] = 'l'
        for obs in self.data:
            r = []
            for c in self.cols:
                y = obs.get(c, '')
                if type(y) is list:
                    y = ','.join(y)

                # http://stackoverflow.com/questions/3224268/python-unicode-encode-error
                # http://stackoverflow.com/questions/19833440/unicodeencodeerror-ascii-codec-cant-encode-character-u-xe9-in-position-7
                try:
                    if type(y) is unicode:
                        y = y.encode('ascii', 'ignore')
                except NameError:
                    # python3
                    pass

                y = str(y)
                y = (y[:self.max_field_size] + '..') if (c != 'altid' and len(y) > self.max_field_size) else y
                r.append(y)
            t.add_row(r)
        return str(t)



from cifsdk.format.cifbind import Bind

import re


def test_format_bind():
    data = [
        {
            'observable': "example.com",
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z',
            'otype': 'fqdn'
        },
        {
            'observable': "example2.com",
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z',
            'otype': 'fqdn'
        },
        {
            'observable': "example3.com",
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z',
            'otype': 'fqdn'
        },
    ]

    text = str(Bind(data))
    assert re.findall(r'^// generated by: CIF at \S+', text)
    assert re.findall(r'\nzone "example.com" {type master; file "\S+";};\n', text)
    assert re.findall(r'\nzone "example3.com" {type master; file "\S+";};', text)

if __name__ == '__main__':
    test_format_bind()


STIX_ENABLED = True

try:
    from stix.core import STIXHeader
    from cifsdk.format.cif_stix import Stix
except ImportError:
    STIX_ENABLED = False

data = [
    {
        'observable': "example.com",
        'provider': "me.com",
        'tlp': "amber",
        'confidence': "85",
        'reporttime': '2015-01-01T00:00:00Z',
        'tags': ['botnet', 'malware']
    },
    {
        'observable': "example.com",
        'provider': "me.com",
        'tlp': "amber",
        'confidence': "85",
        'reporttime': '2015-01-01T00:00:00Z',
        'tags': ['botnet', 'malware']
    },
    {
        'observable': "example.com",
        'provider': "me.com",
        'tlp': "amber",
        'confidence': "85",
        'reporttime': '2015-01-01T00:00:00Z',
        'tags': ['botnet', 'malware']
    }
]


def test_stix():
    if STIX_ENABLED:
        d = Stix(data)
        assert len(str(d)) > 2
    else:
        print('STIX package not installed, skipping test')



try:
    import elasticsearch  # 1.9.0
except ImportError:
    print('this module requires the elasticsearch library')
    raise ImportError

from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter
import logging
import textwrap
from elasticsearch import Elasticsearch
from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta
from elasticsearch import helpers
import re
from cifsdk import VERSION
from cifsdk.utils import setup_logging

CONFIDENCE = 0
MONTHS = 12

from pprint import pprint

def main():

    p = ArgumentParser(
        description=textwrap.dedent('''\
        example usage:
            $ cif-es-reindex
        '''),
        formatter_class=RawDescriptionHelpFormatter,
        prog='cif-es-reindex'
    )

    # options
    p.add_argument("-v", "--verbose", action="store_true", help="logging level: INFO")
    p.add_argument('-d', '--debug', action="store_true", help="logging level: DEBUG")
    p.add_argument('-V', '--version', action='version', version=VERSION)
    p.add_argument('-m', '--months', help='how many months ago to cull [default %(default)s]', default=MONTHS)
    p.add_argument('-c', '--confidence', help='min confidence [default %(default)s]', default=CONFIDENCE)
    p.add_argument('--index-prefix', help='index prefix', default='cif.observables')
    p.add_argument('--dry-run', action="store_true", help='dry run, do not delete')
    p.add_argument('--nodes', default=['localhost:9200'])

    args = p.parse_args()
    setup_logging(args)
    logger = logging.getLogger(__name__)

    end_month = (datetime.today() - relativedelta(months=int(args.months)))
    end_month = end_month.strftime('%Y.%m')

    logger.info('month: {}'.format(end_month))

    es = Elasticsearch(args.nodes, timeout=120, max_retries=10, retry_on_timeout=True)

    # get list of dailies
    dailies = es.indices.get_aliases(index='{}-*.*.*'.format(args.index_prefix)).keys()  # daily indices only

    to_cull = {}
    for d in dailies:
        match = re.search(r"^cif\.observables\-((\d{4}\.\d{2})\.\d{2})$", d)
        if match.group(1):
            if match.group(1) < end_month:
                to_cull['{}-{}'.format(args.index_prefix, match.group(1))] = '{}-{}'.format(args.index_prefix, match.group(2))
    body = {
        'query': {
            'filtered': {
                'filter': {
                    'and': [
                        {
                            'range': {
                                'confidence': {'gte': args.confidence}
                            }
                        }
                    ]
                }
            }
        }
    }

    for c in to_cull:
        logger.info('culling: {}'.format(c))
        if not args.dry_run:
            s, f = helpers.reindex(es, c, target_index=to_cull[c], query=body, chunk_size=50000)
            logger.info('success: {}'.format(s))
            logger.info('failure: {}'.format(f))
            if f:
                logger.error('re-index failed: {}'.format(c))
                raise SystemError
            else:
                logger.info('closing index: {}'.format(c))
                es.indices.close(index=c)

        logger.info('optimizing: {}'.format(to_cull[c]))
        if not args.dry_run:
            es.indices.optimize(index=to_cull[c])



if __name__ == "__main__":
    main()



from cifsdk.format.bro import Bro


def test_format_bro():
    data = [
        {
            'observable': "example.com",
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z',
            'otype': 'fqdn'
        },
        {
            'observable': "http://example.com/1234.htm",
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z',
            'otype': 'url',
        },
        {
            'observable': "https://example.com/1234.htm",
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z',
            'otype': 'url',
        },
        {
            'observable': "192.168.1.1",
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z',
            'otype': 'ipv4'
        }
    ]

    text = str(Bro(data))
    print(text)
    assert text

if __name__ == '__main__':
    test_format_bro()



from cifsdk.format.table import Table


def test_format_table():
    data = [
        {
            'observable': "example.com",
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z'
        },
        {
            'observable': "example.com",
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z'
        },
        {
            'observable': "example.com",
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z'
        }
    ]

    print(Table(data))
    assert Table(data)

if __name__ == '__main__':
    test_format_table()



from cifsdk.format.cifcsv import Csv


def test_format_csv():
    data = [
        {
            'observable': "example.com",
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z',
            'tags': ['botnet', 'malware']
        },
            {
            'observable': "example.com",
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z',
            'tags': ['botnet', 'malware']
        },
        {
            'observable': "example.com",
            'provider': "me.com",
            'tlp': "amber",
            'confidence': "85",
            'reporttime': '2015-01-01T00:00:00Z',
            'tags': ['botnet', 'malware']
        }
    ]
    assert str(Csv(data))


import pytest


from cifsdk.feed.email import Email

data = [
    {
        'observable': 'barbtrave@usamedia.tv',
        'tags': 'phishing',
    },
    {
        'observable': 'bad@example.com',
        'tags': 'malware',
    },
]

whitelist = [
    {
        'observable': 'bad@example.com',
    }
]


def test_feed_email_whitelist():
    x = Email()

    y = x.process(data, whitelist)

    s = set()

    for xx in x.process(data, whitelist):
        s.add(xx['observable'])

    assert 'bad@example.com' not in s
    assert 'barbtrave@usamedia.tv' in s



from pprint import pprint

PERM_WHITELIST = [
    'google.com',
    'yahoo.com',
    'facebook.com',
    'youtube.com',
    'netflix.com',
    'baidu.com',
    'wikipedia.org',
    'twitter.com',
    'qq.com',
    'taobao.com',
    'amazon.com',
    'live.com',
    'bing.com',
    'wordpress.com',
    'msn.com',
    'update.symantec.com',
]


class Fqdn(object):

    def __init__(self):
        self.wl = set()
        for w in PERM_WHITELIST:
            self.wl.add(w)

    def match_whitelist(self, wl, d):
        bits = d.split('.')
        bits2 = list(bits)

        for i, b in enumerate(bits2):
            if '.'.join(bits) in wl:
                return True
            bits.pop(0)

    # https://github.com/jsommers/pytricia
    def process(self, data, whitelist):

        wl = self.wl

        for w in whitelist:
            wl.add(w['observable'])

        rv = []
        for x in data:
            if not self.match_whitelist(wl, x['observable']):
                rv.append(x)

        return rv






from cifsdk.constants import FIELDS
MAX_FIELD_SIZE = 30


class Plugin(object):

    def __init__(self, data=[], cols=FIELDS, max_field_size=MAX_FIELD_SIZE):
        self.cols = cols
        self.max_field_size = max_field_size
        self.data = data
        if self.data:
            if type(self.data) is not list:
                self.data = [self.data]
        else:
            self.data = []

    def __repr__(self):
        raise NotImplementedError


from setuptools import setup, find_packages
import sys

with open('requirements.txt') as f:
    reqs = f.read().splitlines()

import versioneer
versioneer.VCS = 'git'
versioneer.versionfile_source = 'cifsdk/_version.py'
versioneer.versionfile_build = 'cifsdk/_version.py'
versioneer.tag_prefix = ''  # tags are like 1.2.0
versioneer.parentdir_prefix = 'py-cifsdk-'  # dirname like 'myproject-1.2.0'

cmds = [
    "cif=cifsdk.client:main",
]

try:
    import cgmail
    cmds.append('cgmail-cif=cifsdk.cifcgmail:main')
except ImportError:
    pass

try:
    import elasticsearch
    cmds.append('cif-es-reindex=cifsdk.extras.es_reindex:main')
except ImportError:
    pass

setup(
    name="cifsdk",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description="CIF Python SDK",
    long_description="CIF Software Development Kit for Python",
    url="https://github.com/csirtgadgets/cif-sdk-py",
    license='LGPL3',
    classifiers=[
       "Topic :: System :: Networking",
       "Environment :: Other Environment",
       "Intended Audience :: Developers",
       "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
       "Programming Language :: Python",
       ],
    keywords=['cif', ' security'],
    author="Wes Young",
    author_email="wes@barely3am.com",
    packages=find_packages(),
    install_requires=reqs,
    entry_points={
          'console_scripts': cmds
      },
    test_suite="test"
)



from cifsdk.format.plugin import Plugin
import os

SID = os.environ.get('CIF_SNORT_SID', 5000000000)
THRESHOLD = os.environ.get('CIF_SNORT_THRESHOLD', 'type limit,track by_src,count 1,seconds 3600')
SRC = os.environ.get('CIF_SNORT_SRC', 'any')
DEST = os.environ.get('CIF_SNORT_DST', 'any')
MSG_PREFIX = os.environ.get('CIF_SNORT_MSG_PREFIX', 'CIF')
TLP_DEFAULT = os.environ.get('CIF_SNORT_TLP', 'GREEN')
PRIORITY = os.environ.get('CIF_SNORT_PRIOIRTY', 1)
CLASSTYPE = os.environ.get('CIF_SNORT_CLASSTYPE', False)
TAG = os.environ.get('CIF_SNORT_TAG', False)

PROTOCOL_MAP = {
    '0': 'ICMP',
    '6': 'TCP',
    '17': 'UDP'
}


# https://github.com/csirtgadgets/p5-cif-sdk/blob/master/lib/CIF/SDK/Format/Snort.pm
# https://github.com/csirtgadgets/p5-snort-rule/blob/master/lib/Snort/Rule.pm
class Snort(Plugin):
    __name__ = 'cifsnort'

    def __init__(self, *args, **kwargs):
        super(Snort, self).__init__(*args, **kwargs)

        self.cols = ['observable', 'otype', 'tags', 'confidence', 'provider']

    def _dict_to_rule(self, rule, opts=False):
        r = ' '.join([
            rule['action'],
            rule['proto'],
            rule['src'],
            rule['sport'],
            rule['dir'],
            rule['dst'],
            rule['dport'],
        ])

        if opts:
            opstring = '; '.join('{}: {}'.format(v, opts[v]) for v in opts if opts[v])
            r = '{} ({};)'.format(r, opstring)

        return r

    def __repr__(self):
        text = []
        sid = SID
        for d in self.data:
            p = 'IP'
            if d.get('protocol') and d['protocol'] >= 0:
                p = PROTOCOL_MAP.get(str(d['protocol']))

            portlist = 'any'
            if d.get('portlist') and d['portlist'] is not None:
                porlist = str(d['portlist'])

            r = {
                'action': 'alert',
                'proto': p,
                'src': SRC,
                'sport': 'any',
                'dir': '->',
                'dst': d['observable'],
                'dport': portlist,
            }

            opts = {
                'msg': '{} - {} - {}'.format(MSG_PREFIX, TLP_DEFAULT, ','.join(d['tags'])),
                'sid': sid,
                'threshold': THRESHOLD,
                'classtype': CLASSTYPE,
                'reference': d.get('altid', ''),
                'priority': PRIORITY,
                'tag': TAG,

            }

            if d['otype'] == 'ipv4':
                pass

            if d['otype'] == 'ipv4':
                pass

            if d['otype'] == 'fqdn':
                pass

            if d['otype'] == 'url':
                pass

            text.append(self._dict_to_rule(r, opts))
            sid += 1

        return "\n".join(text)



from cifsdk.format.plugin import Plugin
import time

OUTPUT_FILE = '/etc/namedb'


class Bind(Plugin):

    def __init__(self, *args, **kwargs):
        super(Bind, self).__init__(*args, **kwargs)

        self.output = kwargs.get('output', OUTPUT_FILE)

    def __repr__(self):
        text = [
            '// generated by: {} at {}'.format('CIF', time.strftime('%Y-%M-%dT%H:%m:%S %Z'))
        ]
        for obs in self.data:
            if obs['otype'] is not 'fqdn':
                pass

            text.append('zone "{}" {{type master; file "{}";}};'.format(obs['observable'], self.output))

        return '\n'.join(text)


import pytest


from cifsdk.feed.ipv4 import Ipv4
from cifsdk.client import Client
from datetime import datetime
from pprint import pprint

data = [
    {
        'observable': '192.168.1.1',
        'tags': 'botnet',
        'reporttime': datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ'),
        'confidence': 85
    },
    {
        'observable': '192.168.2.1',
        'tags': 'malware',
        'reporttime': datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ'),
        'confidence': 85
    },
    {
        'observable': '128.205.1.1',
        'tags': 'scanner',
        'reporttime': datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ'),
        'confidence': 85
    },
    {
        'observable': '128.205.1.1',
        'tags': 'scanner',
        'reporttime': datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ'),
        'confidence': 75
    },
    {
        'observable': '128.205.2.0/24',
        'tags': 'scanner',
        'reporttime': datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ'),
        'confidence': 85
    },
    {
        'observable': '184.168.047.225/32',
        'tags': 'scanner',
        'reporttime': datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ'),
        'confidence': 85
    },
    {
        'observable': '1.0.0.0/1',
        'tags': 'hijacked',
        'reporttime': datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ'),
        'confidence': 85
    },
    {
        'observable': '1.0.0.0/8',
        'tags': 'hijacked',
        'reporttime': datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ'),
        'confidence': 85
    }
]

whitelist = [
    {
        'observable': '192.168.1.0/24',
        'tags': 'whitelist',
        'reporttime': datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ'),
        'confidence': 85
    },
    {
        'observable': '184.168.047.225/32',
        'tags': 'whitelist',
        'reporttime': datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ'),
        'confidence': 85
    },
]


def test_feed_ipv4_whitelist():
    x = Ipv4()

    s = set()
    for xx in x.process(data, whitelist):
        s.add(xx['observable'])

    assert '192.168.1.1' not in s
    assert '192.168.1.1' not in s
    assert '128.205.1.1' in s

    assert '1.0.0.0/1' not in s

    assert '1.0.0.0/8' in s


def test_feed_ipv4_confidence():
    x = Ipv4()
    cli = Client('http://localhost:5000', 1234)

    r = cli.aggregate(data, field='observable')
    r = x.process(r, whitelist)

    s = set()
    for rr in r:
        if rr['observable'] not in s:
            s.add(rr['confidence'])

    assert 75 not in s



import py.test
from cifsdk.client import Client


def test_client():
    cli = Client(token=1234, remote='https://localhost2:8443', verify_ssl=False)

    assert cli.verify_ssl is False
    assert cli.remote == 'https://localhost2:8443','remote-mismatch'
    assert cli.token == '1234', 'token mismatch'



#!/usr/bin/env python

try:
    import cgmail
except ImportError:
    print('py-cgmail needs to be installed: https://github.com/csirtgadgets/py-cgmail')

from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter
import logging
import os.path
import textwrap
import sys
from pprint import pprint

from cifsdk.utils import setup_logging, read_config
from cifsdk.client import Client
from cifsdk.observable import Observable
import re
import arrow
from cgmail.urls import url_to_fqdn

try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse

LOG_FORMAT = '%(asctime)s - %(levelname)s - %(name)s[%(lineno)s] - %(message)s'
CONFIDENCE = 50
TLP = 'green'
PROVIDER = 'localhost'

EXCLUDE = os.environ.get('CGMAIL_EXCLUDE', None)
CONFIDENCE_LOWER = os.environ.get('CGMAIL_CONFIDENCE_LOWER', None)
WHITELIST_CACHE = os.environ.get('CGMAIL_WHITELIST_CACHE', '/tmp/cgmail_whitelist.txt')
BLACKLIST_CACHE = os.environ.get('CGMAIL_BLACKLIST_CACHE', '/tmp/cgmail_blacklist.txt')

URL_SHORTNERS = (
    'tinyurl.com',
    'goo.gl',
)

HOSTING_PROVIDERS = (
    'esy.es'
)


def match_whitelist(wl, d):
    bits = d.split('.')
    bits2 = list(bits)

    for i, b in enumerate(bits2):
        if '.'.join(bits) in wl:
            return True
        bits.pop(0)


def main():

    #
    # initialize module
    #

    p = ArgumentParser(
        description=textwrap.dedent('''\
        example usage:
            $ cat test.eml | cgmail
            $ cgmail --file test.eml
        '''),
        formatter_class=RawDescriptionHelpFormatter,
        prog='cgmail'
    )

    p.add_argument("-v", "--verbose", dest="verbose", action="count",
                   help="set verbosity level [default: %(default)s]")
    p.add_argument('-d', '--debug', dest='debug', action="store_true")

    p.add_argument("-f", "--file", dest="file", help="specify email file")

    # cif arguments
    p.add_argument("--confidence", help="specify confidence for submitting to CIF", default=CONFIDENCE)
    p.add_argument("--remote", help="specify CIF remote")
    p.add_argument("--token", help="specify CIF token")
    p.add_argument("--config", help="specify CIF config [default: %(default)s",
                   default=os.path.expanduser("~/.cif.yml"))
    p.add_argument("--tags", help="specify CIF tags [default: %(default)s", default=["phishing"])
    p.add_argument("--group", help="specify CIF group [default: %(default)s", default="everyone")
    p.add_argument("--tlp", help="specify CIF TLP [default: %(default)s", default=TLP)
    p.add_argument("--no-verify-ssl", action="store_true", default=False)
    p.add_argument("--raw", action="store_true", help="include raw message data")
    p.add_argument("--provider", help="specify feed provider [default: %(default)s]", default=PROVIDER)

    p.add_argument('--exclude', help='url patterns to exclude [default: %(default)s', default=EXCLUDE)
    p.add_argument('--confidence-lower', help='patterns to automatically lower confidence', default=CONFIDENCE_LOWER)
    p.add_argument('-n', '--not-really', help='do not submit', action='store_true')
    p.add_argument('--cache', help='location to cache whitelist [default: %(default)s', default=WHITELIST_CACHE)
    p.add_argument('--blacklist-cache', default=BLACKLIST_CACHE)

     # Process arguments
    args = p.parse_args()
    setup_logging(args)
    logger = logging.getLogger(__name__)

    exclude = None
    if args.exclude:
        exclude = re.compile(args.exclude)

    confidence_lower = None
    if args.confidence_lower:
        confidence_lower = re.compile(args.confidence_lower)

    o = read_config(args)
    options = vars(args)
    for v in options:
        if options[v] is None:
            options[v] = o.get(v)

    if not options.get('token'):
        raise RuntimeError('missing --token')

    if options.get("file"):
        with open(options["file"]) as f:
            email = f.read()
    else:
        email = sys.stdin.read()

    # extract urls from message body and mail parts
    bits = cgmail.parse_email_from_string(email)
    urls = set()

    for n in bits:
        if n.get('urls'):
            for u in n['urls']:
                urls.add(u)

    verify_ssl = True
    if options.get('no_verify_ssl'):
        verify_ssl = False

    # initialize cif client
    cli = Client(remote=options["remote"], token=options["token"], verify_ssl=verify_ssl)

    update_cache = True
    if os.path.isfile(args.cache):
        modified = os.path.getmtime(args.cache)
        if arrow.utcnow() < arrow.get(modified + 84600):
            update_cache = False

    if update_cache:
        # pull FQDN whitelist

        filters = {
            'tags': 'whitelist',
            'otype': 'fqdn',
            'confidence': 25,
        }
        now = arrow.utcnow()
        filters['reporttimeend'] = '{0}Z'.format(now.format('YYYY-MM-DDTHH:mm:ss'))
        now = now.replace(days=-7)
        filters['reporttime'] = '{0}Z'.format(now.format('YYYY-MM-DDTHH:mm:ss'))

        ret = cli.search(limit=50000, filters=filters, sort='reporttime', sort_direction='desc')
        with open(args.cache, 'w') as f:
            for r in ret:
                f.write("{0}\n".format(r['observable']))

    update_cache = True
    if os.path.isfile(args.blacklist_cache):
        modified = os.path.getmtime(args.blacklist_cache)
        if arrow.utcnow() < arrow.get(modified + 84600):
            update_cache = False

    if update_cache:
        filters = {
            'tags': 'phishing,suspicious,malware',
            'otype': 'fqdn',
            'confidence': 75,
        }
        now = arrow.utcnow()
        filters['reporttimeend'] = '{0}Z'.format(now.format('YYYY-MM-DDTHH:mm:ss'))
        now = now.replace(days=-7)
        filters['reporttime'] = '{0}Z'.format(now.format('YYYY-MM-DDTHH:mm:ss'))

        ret = cli.search(limit=50000, filters=filters, sort='reporttime', sort_direction='desc')
        with open(args.blacklist_cache, 'w') as f:
            for r in ret:
                f.write("{0}\n".format(r['observable']))

    fqdns = set()
    with open(args.cache) as f:
        for l in f:
            fqdns.add(l.rstrip("\n"))

    fqdns_blacklist = set()
    with open(args.blacklist_cache) as f:
        for l in f:
            fqdns_blacklist.add(l.rstrip("\n"))

    for u in urls:
        u = u.rstrip('\/')
        u = urlparse(u)

        fqdn = url_to_fqdn(u.geturl())
        if exclude and exclude.search(fqdn):
            continue

        confidence = options['confidence']

        if match_whitelist(fqdns, u.netloc):
            if (u.netloc not in URL_SHORTNERS) and (not match_whitelist(HOSTING_PROVIDERS, u.netloc)):
                confidence = options['confidence'] - 15
            else:
                confidence = options['confidence'] + 5
        elif match_whitelist(fqdns_blacklist, u.netloc):
            confidence = options['confidence'] + 10
        else:
            confidence = options['confidence'] + 5

        # else
        # raise confidence
        logger.info("submitting: {0}".format(u.geturl()))

        o = Observable(
            observable=u.geturl(),
            confidence=confidence,
            tlp=options["tlp"],
            group=options["group"],
            tags=options["tags"],
            provider=options.get('provider')
        )

        o = o.__dict__
        del o['logger']

        if options.get('raw'):
            o.raw = email

        if not args.not_really:
            r = cli.submit(o)
            logger.info("submitted: {0}".format(r))


if __name__ == "__main__":
    main()
